@@ -1,5 +1,5 @@
 /*
- * SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * SPDX-FileCopyrightText: Copyright (c) 2022-2025 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * SPDX-License-Identifier: MIT
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
@@ -50,6 +50,7 @@
 #include "gpu/bus/kern_bus.h"
 #include "gpu/mem_mgr/mem_desc.h"
 #include "gpu/mem_mgr/mem_mgr.h"
+#include "gpu/mem_sys/kern_mem_sys.h"
 #include "gpu/subdevice/subdevice.h"
 #include "kernel/gpu/nvlink/kernel_nvlink.h"
 #include "mem_mgr/fabric_vaspace.h"
@@ -76,14 +77,22 @@ typedef struct mem_multicast_fabric_client_info
     Memory *pMemory;
 } MEM_MULTICAST_FABRIC_CLIENT_INFO;
 
+typedef struct mem_multicast_fabric_gpu_nvlink_attr
+{
+    NvU32   cliqueId;
+    NvU64   bwModeEpoch;
+    NvU8    bwMode;
+} MEM_MULTICAST_FABRIC_GPU_NVLINK_ATTR;
+
 typedef struct mem_multicast_fabric_gpu_info
 {
     void   *pGpuOsInfo;
     OBJGPU *pGpu;
     NvU64   gpuProbeHandle;
-    NvU32   cliqueId;
     NvBool  bMcflaAlloc;
 
+    MEM_MULTICAST_FABRIC_GPU_NVLINK_ATTR nvlAttr;
+
     //
     // Unique import event ID. Valid only if the GPU was remotely attached to
     // the prime MCFLA object
@@ -100,7 +109,8 @@ typedef struct mem_multicast_fabric_gpu_info
 
 typedef struct mem_multicast_fabric_remote_gpu_info
 {
-    NvU32 cliqueId;
+    MEM_MULTICAST_FABRIC_GPU_NVLINK_ATTR nvlAttr;
+
     NvU64 key;
 } MEM_MULTICAST_FABRIC_REMOTE_GPU_INFO;
 
@@ -233,6 +243,8 @@ _memMulticastFabricInitAttachEvent
 (
     NvU64                        gpuFabricProbeHandle,
     NvU64                        key,
+    NvU64                        bwModeEpoch,
+    NvU8                         bwMode,
     NvU32                        cliqueId,
     NvU16                        exportNodeId,
     NvU16                        index,
@@ -250,6 +262,8 @@ _memMulticastFabricInitAttachEvent
     pEvent->data.attach.gpuFabricProbeHandle = gpuFabricProbeHandle;
     pEvent->data.attach.key = key;
     pEvent->data.attach.cliqueId = cliqueId;
+    pEvent->data.attach.bwModeEpoch = bwModeEpoch;
+    pEvent->data.attach.bwMode = bwMode;
     pEvent->data.attach.index = index;
     pEvent->data.attach.exportNodeId = exportNodeId;
     portMemCopy(pEvent->data.attach.exportUuid, NV_MEM_EXPORT_UUID_LEN,
@@ -951,6 +965,7 @@ _memMulticastFabricSendInbandRequest
 static void
 _memorymulticastfabricDetachMem
 (
+    OBJGPU             *pGpu,
     FABRIC_VASPACE     *pFabricVAS,
     MEMORY_DESCRIPTOR  *pFabricMemDesc,
     NODE               *pMemNode
@@ -967,6 +982,7 @@ _memorymulticastfabricDetachMem
                                   pMemNode->keyStart,
                                   pPhysMemDesc,
                                   pAttachMemInfoNode->physMapLength);
+
 }
 
 static void
@@ -1764,6 +1780,70 @@ memorymulticastfabricConstruct_IMPL
                                         pCallContext, pParams);
 }
 
+static NV_STATUS
+_memorymulticastfabricValidateNvlAttrCommon
+(
+    MEM_MULTICAST_FABRIC_GPU_NVLINK_ATTR *pNvlAttr,
+    NvU32                                 cliqueId,
+    NvU8                                  bwMode,
+    NvU64                                 bwModeEpoch
+)
+{
+    if (pNvlAttr->cliqueId != cliqueId)
+    {
+        NV_PRINTF(LEVEL_ERROR, "Clique ID mismatch %u:%u\n", pNvlAttr->cliqueId, cliqueId);
+        return NV_ERR_INVALID_DEVICE;
+    }
+
+    if (pNvlAttr->bwModeEpoch != bwModeEpoch)
+    {
+        NV_PRINTF(LEVEL_ERROR, "bwModeEpoch mismatch %llu:%llu\n", pNvlAttr->bwModeEpoch, bwModeEpoch);
+        return NV_ERR_INVALID_DEVICE;
+    }
+
+    if (pNvlAttr->bwMode != bwMode)
+    {
+        NV_PRINTF(LEVEL_ERROR, "bwMode mismatch %u:%u\n", pNvlAttr->bwMode, bwMode);
+        return NV_ERR_INVALID_DEVICE;
+    }
+
+    return NV_OK;
+}
+
+static NV_STATUS
+_memorymulticastfabricValidateNvlAttr
+(
+    MEM_MULTICAST_FABRIC_DESCRIPTOR *pMulticastFabricDesc,
+    NvU32                            cliqueId,
+    NvU8                             bwMode,
+    NvU64                            bwModeEpoch
+)
+{
+    MEM_MULTICAST_FABRIC_GPU_INFO *pHead;
+    MEM_MULTICAST_FABRIC_REMOTE_GPU_INFO *pRemoteHead;
+
+    pHead = listHead(&pMulticastFabricDesc->gpuInfoList);
+    pRemoteHead = multimapFirstItem(&pMulticastFabricDesc->remoteGpuInfoMap);
+
+    if (pHead != NULL)
+    {
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+            _memorymulticastfabricValidateNvlAttrCommon(&pHead->nvlAttr,
+                                                        cliqueId, bwMode,
+                                                        bwModeEpoch));
+    }
+
+    if (pRemoteHead != NULL)
+    {
+        NV_CHECK_OK_OR_RETURN(LEVEL_ERROR,
+            _memorymulticastfabricValidateNvlAttrCommon(&pRemoteHead->nvlAttr,
+                                                        cliqueId, bwMode,
+                                                        bwModeEpoch));
+    }
+
+    return NV_OK;
+}
+
 //
 // Note this function is not always called with the GPU lock. Be careful
 // while accessing GPU state.
@@ -1781,8 +1861,6 @@ _memorymulticastfabricCtrlAttachGpu
     NV_STATUS status;
     FABRIC_VASPACE *pFabricVAS;
     MEM_MULTICAST_FABRIC_GPU_INFO *pNode;
-    MEM_MULTICAST_FABRIC_GPU_INFO *pHead;
-    MEM_MULTICAST_FABRIC_REMOTE_GPU_INFO *pRemoteHead;
     CALL_CONTEXT *pCallContext = resservGetTlsCallContext();
 
     if (pParams->flags != 0)
@@ -1834,7 +1912,7 @@ _memorymulticastfabricCtrlAttachGpu
     }
 
     status = gpuFabricProbeGetFabricCliqueId(pGpu->pGpuFabricProbeInfoKernel,
-                                             &pNode->cliqueId);
+                                             &pNode->nvlAttr.cliqueId);
     if (status != NV_OK)
     {
         NV_PRINTF(LEVEL_ERROR,
@@ -1842,20 +1920,18 @@ _memorymulticastfabricCtrlAttachGpu
         goto fail;
     }
 
-    pHead = listHead(&pMulticastFabricDesc->gpuInfoList);
-    pRemoteHead = multimapFirstItem(&pMulticastFabricDesc->remoteGpuInfoMap);
+    pNode->nvlAttr.bwMode = knvlinkGetBWMode(pGpu, GPU_GET_KERNEL_NVLINK(pGpu));
 
-    if ((pHead != NULL) && (pHead->cliqueId != pNode->cliqueId))
-    {
-        NV_PRINTF(LEVEL_ERROR, "Clique ID mismatch\n");
-        status = NV_ERR_INVALID_DEVICE;
-        goto fail;
-    }
+    pNode->nvlAttr.bwModeEpoch = knvlinkGetBWModeEpoch(pGpu,
+                                                       GPU_GET_KERNEL_NVLINK(pGpu));
 
-    if ((pRemoteHead != NULL) && (pRemoteHead->cliqueId != pNode->cliqueId))
+    status = _memorymulticastfabricValidateNvlAttr(pMulticastFabricDesc,
+                                                   pNode->nvlAttr.cliqueId,
+                                                   pNode->nvlAttr.bwMode,
+                                                   pNode->nvlAttr.bwModeEpoch);
+    if (status != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR, "Clique ID mismatch\n");
-        status = NV_ERR_INVALID_DEVICE;
+        NV_PRINTF(LEVEL_ERROR, "Clique ID etc. validation failed\n");
         goto fail;
     }
 
@@ -1915,7 +1991,9 @@ _memorymulticastfabricCtrlAttachGpu
 
         _memMulticastFabricInitAttachEvent(pNode->gpuProbeHandle,
                                            pMulticastFabricDesc->inbandReqId,
-                                           pNode->cliqueId,
+                                           pNode->nvlAttr.bwModeEpoch,
+                                           pNode->nvlAttr.bwMode,
+                                           pNode->nvlAttr.cliqueId,
                                            pMulticastFabricDesc->exportNodeId,
                                            pMulticastFabricDesc->index,
                                            &pMulticastFabricDesc->expUuid,
@@ -2116,10 +2194,6 @@ _memorymulticastfabricCtrlAttachRemoteGpu
 {
     MEM_MULTICAST_FABRIC_DESCRIPTOR *pMulticastFabricDesc =
                                 pMemoryMulticastFabric->pMulticastFabricDesc;
-    MEM_MULTICAST_FABRIC_GPU_INFO *pHead =
-                        listHead(&pMulticastFabricDesc->gpuInfoList);
-    MEM_MULTICAST_FABRIC_REMOTE_GPU_INFO *pRemoteHead =
-                    multimapFirstItem(&pMulticastFabricDesc->remoteGpuInfoMap);
     MEM_MULTICAST_FABRIC_REMOTE_GPU_INFO *pNode;
     MEM_MULTICAST_FABRIC_GPU_INFO *pIter;
     MemMulticastFabricRemoteGpuInfoMapSubmap *pSubmap = NULL;
@@ -2152,17 +2226,13 @@ _memorymulticastfabricCtrlAttachRemoteGpu
         goto fail;
     }
 
-    if ((pHead != NULL) && (pHead->cliqueId != pParams->cliqueId))
+    status = _memorymulticastfabricValidateNvlAttr(pMulticastFabricDesc,
+                                                   pParams->cliqueId,
+                                                   pParams->bwMode,
+                                                   pParams->bwModeEpoch);
+    if (status != NV_OK)
     {
-        NV_PRINTF(LEVEL_ERROR, "Clique ID mismatch\n");
-        status = NV_ERR_INVALID_DEVICE;
-        goto fail;
-    }
-
-    if ((pRemoteHead != NULL) && (pRemoteHead->cliqueId != pParams->cliqueId))
-    {
-        NV_PRINTF(LEVEL_ERROR, "Clique ID mismatch\n");
-        status = NV_ERR_INVALID_DEVICE;
+        NV_PRINTF(LEVEL_ERROR, "Clique ID etc. validation failed\n");
         goto fail;
     }
 
@@ -2211,7 +2281,9 @@ _memorymulticastfabricCtrlAttachRemoteGpu
     portMemSet(pNode, 0, sizeof(*pNode));
 
     pNode->key = pParams->key;
-    pNode->cliqueId = pParams->cliqueId;
+    pNode->nvlAttr.cliqueId = pParams->cliqueId;
+    pNode->nvlAttr.bwMode = pParams->bwMode;
+    pNode->nvlAttr.bwModeEpoch = pParams->bwModeEpoch;
 
     if ((pMulticastFabricDesc->numAttachedGpus + 1)
                                     == pMulticastFabricDesc->numMaxGpus)
@@ -2416,7 +2488,7 @@ _memorymulticastfabricCtrlDetachMem
     pFabricVAS = dynamicCast(pGpuInfo->pGpu->pFabricVAS, FABRIC_VASPACE);
     NV_ASSERT_OR_RETURN(pFabricVAS != NULL, NV_ERR_INVALID_STATE);
 
-    _memorymulticastfabricDetachMem(pFabricVAS, pFabricMemDesc, pNode);
+    _memorymulticastfabricDetachMem(pGpu, pFabricVAS, pFabricMemDesc, pNode);
 
     if (bRemoveInterMapping)
     {
@@ -2514,7 +2586,7 @@ _memorymulticastfabricValidatePhysMem
     {
         NV_PRINTF(LEVEL_ERROR, "Device-less memory isn't supported yet\n");
 
-        return NV_ERR_NOT_SUPPORTED;
+        return NV_ERR_INVALID_ARGUMENT;
     }
 
     if (!memmgrIsMemDescSupportedByFla_HAL(pPhysMemDesc->pGpu,
@@ -2564,7 +2636,6 @@ _memorymulticastfabricCtrlAttachMem
     MEM_MULTICAST_FABRIC_ATTACH_MEM_INFO_NODE *pNode;
     NvU32 gpuMask = NVBIT(gpuGetInstance(pGpu));
     RsInterMapping *pInterMapping;
-
     if (!rmGpuGroupLockIsOwner(0, GPU_LOCK_GRP_MASK, &gpuMask))
         return NV_ERR_INVALID_LOCK_STATE;
 
@@ -2620,12 +2691,12 @@ _memorymulticastfabricCtrlAttachMem
 
     portMemSet(pNode, 0, sizeof(*pNode));
 
-    pNode->node.keyStart = pParams->offset;
-    pNode->node.keyEnd   = pParams->offset;
-    pNode->physMapLength = pParams->mapLength;
-    pNode->pPhysMemDesc  = pPhysMemDesc;
-    pNode->pInterMapping = pInterMapping;
-    pNode->node.Data     = pNode;
+    pNode->node.keyStart    = pParams->offset;
+    pNode->node.keyEnd      = pParams->offset;
+    pNode->physMapLength    = pParams->mapLength;
+    pNode->pPhysMemDesc     = pPhysMemDesc;
+    pNode->pInterMapping    = pInterMapping;
+    pNode->node.Data        = pNode;
 
     status = btreeInsert(&pNode->node, &pGpuInfo->pAttachMemInfoTree);
     if (status != NV_OK)
